/*
  merlin_mount.cpp - Arduino library support for Merlin/Orion telescope mounts
  Copyright (c)2009 Gustav Evertsson All right reserved
  www.guzzzt.com - me@guzzzt.com
 
  
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
  Version:   1.0 - Aug 7 2009
  
*/
#include "Arduino.h"

#include "merlin_mount.h"
#include "HardwareSerial.h"
//#include "WConstants.h"		//all things wiring / arduino

  // Init
uint8_t MerlinMount::init() {
	char res[16];
	
	if(this->initiated == 1)
		return INIT_SUCCESS;
	
	this->sendCommand(":F1\r", res);
	this->sendCommand(":F2\r", res);
	this->sendCommand(":a1\r", res);
	this->sendCommand(":D1\r", res);
	this->sendCommand(":a2\r", res);
	this->sendCommand(":D2\r", res);
	this->initiated = 1;
	
	return INIT_SUCCESS;
}

  // Control functions
void MerlinMount::startMoving(uint8_t axis, uint8_t dir) {
	char res[16];

	if(axis == AXIS_YAW) {
	  this->sendCommand(":L1\r", res);
	  if(dir == DIR_RIGHT)
	  	this->sendCommand(":G130\r", res);
	  else
	  	this->sendCommand(":G131\r", res);
	  this->sendCommand(":I1220000\r", res);
	  this->sendCommand(":J1\r", res);
	}
	else {
	  this->sendCommand(":L2\r", res);
	  if(dir == DIR_UP)
	  	this->sendCommand(":G230\r", res);
	  else
	  	this->sendCommand(":G231\r", res);
	  this->sendCommand(":I2220000\r", res);
	  this->sendCommand(":J2\r", res);		
	}
}

void MerlinMount::stopMoving(uint8_t axis) {
	char res[16];

	if(axis == AXIS_YAW) {
	  this->sendCommand(":L1\r", res);
	  this->sendCommand(":f1\r", res);
	}
	else {
	  this->sendCommand(":L2\r", res);
	  this->sendCommand(":f2\r", res);
	}
}

void MerlinMount::driveToPosition(uint8_t axis, long position) {
	char res[16];
	char pos[6];
	char cmd[12];
	long tempPos = 0;
	
	tempPos = this->readAxisPosition(axis) - position;
	if(tempPos < 0)
		tempPos = tempPos * -1;
	if(tempPos < AXIS_ACCURACY)
		return;
	
	valToHexStr(position, pos);
	cmd[0] = ':';
	cmd[1] = 'S';
	cmd[2] = '0' + axis;
	for(int i = 0; i < 6; i++)
		cmd[i+3] = pos[i];
	cmd[9] = '\r';
	
	if(axis == AXIS_YAW) {
	  this->sendCommand(":L1\r", res);
	  this->sendCommand(":G100\r", res);	  
	  this->sendCommand(cmd, res);
	  this->sendCommand(":J1\r", res);
	}
	else {
	  this->sendCommand(":L2\r", res);
	  this->sendCommand(":G200\r", res);	  
	  this->sendCommand(cmd, res);
	  this->sendCommand(":J2\r", res);
	}
	
}

void MerlinMount::driveToPositionBothAxis(long positionYaw, long positionPitch, uint8_t sync) {
	driveToPosition(AXIS_YAW, positionYaw);
	driveToPosition(AXIS_PITCH, positionPitch);
	
	// if synchrony then wait unit both axis have stopped
	if(sync == SYNC_SYNCHRONY) {
		delay(600);
		while(this->readAxisStatus(AXIS_YAW) == STATUS_RUNNING ||
					this->readAxisStatus(AXIS_PITCH) == STATUS_RUNNING) {
			delay(200);
		}
	}

}

  // Camera functions
void MerlinMount::takePicture(int shutterTime) {
	char res[10];
	
	this->sendCommand(":O11\r", res);
	delay(shutterTime);
	this->sendCommand(":O10\r", res);
}

  	
	// Status functions
long MerlinMount::readAxisPosition(uint8_t axis) {
	char res[16];
	char pos[10];
	
	if(axis == AXIS_YAW) {
	  this->sendCommand(":j1\r", res);
	}
	else {
	  this->sendCommand(":j2\r", res);
	}

	for(int i = 0; i < 6; i++)
		pos[i] = res[i+5];


	return parsePosVal(pos);
}

uint8_t MerlinMount::readAxisStatus(uint8_t axis) {
	char res[16];

	if(axis == AXIS_YAW) {
	  this->sendCommand(":f1\r", res);
	}
	else {
	  this->sendCommand(":f2\r", res);
	}

	if(res[6] == '0')
		return STATUS_STOPPED;
	else
		return STATUS_RUNNING;
}

	// Low byte is sent first (A35483 is read 8354A3).
long MerlinMount::parsePosVal(char* hexStr) {
	long val = 0;
	char c = 0;
  for(int i = 5; i > 0; i = i - 2) {
    val <<= 4;
    c = hexStr[i-1];
      
    if( c >= '0' && c <= '9')
      val += (c - '0');
    else if( c >= 'A' && c <= 'F')
      val += (c - 'A' + 10);


    val <<= 4;
    c = hexStr[i];
      
    if( c >= '0' && c <= '9')
      val += (c - '0');
    else if( c >= 'A' && c <= 'F')
      val += (c - 'A' + 10);
  }
   return val;
}

void MerlinMount::valToHexStr(long val, char* hexStr) {
	char hexval[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};

	const byte* p = (const byte*)(const void*)&val;
	
	hexStr[0] = hexval[((*p >> 4) & 0xF)];
	hexStr[1] = hexval[*p & 0x0F];
	p++;
	hexStr[2] = hexval[((*p >> 4) & 0xF)];
	hexStr[3] = hexval[*p & 0x0F];
	p++;
	hexStr[4] = hexval[((*p >> 4) & 0xF)];
	hexStr[5] = hexval[*p & 0x0F];
}

double MerlinMount::toAngle(long first, long second) {
	long distance = 0;
	distance = first - second;
	
	return (double)distance / (double)COMPLETE_TURN * 360.0;
}

long MerlinMount::fromAngle(double angle) {
	return (long)(((double)COMPLETE_TURN / 360.0) * angle);
}

	// Communication with the Merlin mount
void MerlinMount::sendCommand(const char* send, char* respose) {
  uint8_t incomingByte = 0;
  uint8_t pos = 0;
  
  Serial.print(send);
  delay(50);
  while (Serial.available() > 0) {
    // read the incoming byte:
    incomingByte = Serial.read();
    respose[pos++] = incomingByte;
  }
  respose[pos] = '\0';
}

  // class constructor
MerlinMount::MerlinMount() {
  Serial.begin(9600);
  this->initiated = 0;
}

// Make one instance for the user
MerlinMount merlin = MerlinMount();
