/*
  GigaPixel
  Copyright 2009 Gustav Evertsson
  www.gigapixel.nu - info@gigapixel.nu

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Arduino.h"


#include <merlin_mount.h>

#include <ks0108.h>	// Graphical LCD lib
#include <HeaderFont.h>  // font definitions 
#include <NormalFont.h>  // font definitions 
#include <SmallFont.h>  // font definitions 
#include <EEPROM.h>
#include "EEPROM_anything.h"

#include <gigapixel_menu.h>  // MENU Image from the ks0108 lib folder 

// Pin configuration (GLCD pins are defined in the ks0108 lib)
#define BUTTONS_LATCH_PIN  13	// Shift in pin
#define BUTTONS_DATA_PIN   2	// Shift in pin
#define BUTTONS_CLOCK_PIN  3	// Shift in pin
#define SPEEAKER_PIN       12	// Summer output pin

// Time limits
#define BUTTON_DEBOUNCE_TIME 200

// Menus/States
#define MENU_MAIN                   1
#define MENU_CONFIG                 2
#define MENU_CONFIG_PAGE_2          16
#define MENU_CONFIG_ORDER           3
#define MENU_CONFIG_OVERLAP         4
#define MENU_CONFIG_STAB_DELAY      5
#define MENU_CONFIG_BRACKETING      6
#define MENU_CONFIG_SHUTTER_DELAY   15
#define MENU_NEW_SHOT_FOCAL_LENGTH  7
#define MENU_NEW_SHOT_START_POS     8
#define MENU_NEW_SHOT_END_POS       9
#define MENU_NEW_SHOT_READY         10
#define MENU_PROGRESS               11
#define MENU_FINISHED               12
#define MENU_CANCEL                 13
#define MENU_PAUSE                  14

// Sub states used during shooting
#define PROGRESS_START      1
#define PROGRESS_MOVING     2
#define PROGRESS_WAIT       3
#define PROGRESS_SHOOTING   4
#define PROGRESS_WAIT_SHUT  5

// Main menu sub items
#define SUB_MENU_MAIN_NEW_SHOT 1
#define SUB_MENU_MAIN_CONFIG   2

// Config menu sub items
#define SUB_MENU_CONFIG_ORDER      1
#define SUB_MENU_CONFIG_OVERLAP    2
#define SUB_MENU_CONFIG_STAB_DELAY 3
#define SUB_MENU_CONFIG_BRACKETING 4

#define SUB_MENU_CONFIG_SHUT_DELAY 5

// Nikon D90 Sensor Dimensions (mounted in Potrait mode)
#define SENSOR_WIDTH    15.8
#define SENSOR_HEIGHT   23.6

// Values for the config value order
#define ORDER_ROWS     1
#define ORDER_COLUMNS  2

// EEPROM addresses used to save the config
#define EEPROM_INIT_ADDR          0
#define EEPROM_FOCAL_LENGTH_ADDR  1
#define EEPROM_ORDER_ADDR         3
#define EEPROM_OVERLAP_ADDR       5
#define EEPROM_STAB_DELAY_ADDR    7
#define EEPROM_BRACKETING_ADDR    9
#define EEPROM_SHUTTER_DELAY_ADDR 11


// All Button states
boolean btn_enter = 0;
boolean btn_back = 0;
boolean btn_left = 0;
boolean btn_right = 0;
boolean btn_up = 0;
boolean btn_down = 0;

// Global Config Values
int order = ORDER_ROWS;
int overlap = 25;
int stab_delay = 2;
int bracketing = 1;
int shutter_delay = 2;

int oldConfigValue = 0;  // Used to store the original value if the user press "Back" in config menus

// New Shot Config Values
int focal_length = 300;
double focal_length_angle_widht = 0.0;
double focal_length_angle_height = 0.0;
long start_pos_x = 0;
long start_pos_y = 0;
long end_pos_x = 0;
long end_pos_y = 0;
int images_horizontal = 0;
int images_vertical = 0;
int images_total = 0;

int status_yaw = DIR_STOPPED;
int status_pitch = DIR_STOPPED;

// Variables used during shooting.
int progress_state = PROGRESS_START;
int current_image = 1;
int current_horizontal = 0;
int current_vertical = 0;
unsigned long progress_timer = 0;
unsigned long wait_timer = 0;
unsigned long shooting_start_time = 0;
int pause_image = 0;

// Menu
int menu = MENU_MAIN;
int subMenu = 0;
unsigned long menu_debounce_timer = 0;
unsigned long menu_update_timer = 0;

// Success Sound when Finished!
int length = 15; // the number of notes
char notes[] = "ccggaagffeeddc "; // a space represents a rest
int beats[] = { 1, 1, 1, 1, 1, 1, 2, 1, 1, 1, 1, 1, 1, 2, 4 };
int tempo = 300;




void setup(){
  //start serial used to communicate with Merlin/Orion mount
  Serial.begin(9600);
  
  //define pin modes
  pinMode(BUTTONS_LATCH_PIN, OUTPUT);
  pinMode(BUTTONS_CLOCK_PIN, OUTPUT); 
  pinMode(BUTTONS_DATA_PIN, INPUT);
  pinMode(SPEEAKER_PIN, OUTPUT);

	//If the init address is set then it should be save to read the config from EEPROM
	//else set the values to default
  byte eepromInit = 0;
  EEPROM_readAnything(EEPROM_INIT_ADDR, eepromInit);
  if(eepromInit == 1) {
    EEPROM_readAnything(EEPROM_FOCAL_LENGTH_ADDR, focal_length);
    EEPROM_readAnything(EEPROM_ORDER_ADDR, order);
    EEPROM_readAnything(EEPROM_OVERLAP_ADDR, overlap);
    EEPROM_readAnything(EEPROM_STAB_DELAY_ADDR, stab_delay);
    EEPROM_readAnything(EEPROM_BRACKETING_ADDR, bracketing);
    EEPROM_readAnything(EEPROM_SHUTTER_DELAY_ADDR, shutter_delay);
  }
  else {
    //Write default values to the EEPROM
    EEPROM_writeAnything(EEPROM_INIT_ADDR, (byte)1);
    EEPROM_writeAnything(EEPROM_FOCAL_LENGTH_ADDR, focal_length);
    EEPROM_writeAnything(EEPROM_ORDER_ADDR, order);
    EEPROM_writeAnything(EEPROM_OVERLAP_ADDR, overlap);
    EEPROM_writeAnything(EEPROM_STAB_DELAY_ADDR, stab_delay);
    EEPROM_writeAnything(EEPROM_BRACKETING_ADDR, bracketing);
    EEPROM_writeAnything(EEPROM_SHUTTER_DELAY_ADDR, shutter_delay);
  }
  
  GLCD.Init(NON_INVERTED);   // initialise the GLCD library, non inverted writes pixels onto a clear screen
  
  drawMainMenu();	//Start by drawing the main menu
}



void  loop() {   // run over and over again
  updateButtonsState();	//Update info about the button states
  
  switch(menu) {
    case MENU_MAIN:
      loopMainMenu();
      break;
    case MENU_CONFIG:
      loopConfigMenu();
      break;
    case MENU_CONFIG_PAGE_2:
      loopConfigPage2Menu();
      break;
    case MENU_CONFIG_ORDER:
      loopConfigOrder();
      break;
    case MENU_CONFIG_OVERLAP:
      loopConfigOverlap();
      break;
    case MENU_CONFIG_STAB_DELAY:
      loopConfigStabDelay();
      break;
    case MENU_CONFIG_BRACKETING:
      loopConfigBracketing();
      break;
    case MENU_CONFIG_SHUTTER_DELAY:
      loopConfigShutterDelay();
      break;
    case MENU_NEW_SHOT_FOCAL_LENGTH:
      loopNewShotFocalLength();
      break;
    case MENU_NEW_SHOT_START_POS:
      loopNewShotStartPos();
      break;
    case MENU_NEW_SHOT_END_POS:
      loopNewShotEndPos();
      break;
    case MENU_NEW_SHOT_READY:
      loopNewShotReady();
      break;
    case MENU_PROGRESS:
      loopProgress();
      break;
    case MENU_FINISHED:
      loopFinished();
      break;
    case MENU_CANCEL:
      loopCancel();
      break;
    case MENU_PAUSE:
      loopPause();
      break;
  }

}

///////////////////////////////////////////////////////
//                   MAIN MENU                       //
///////////////////////////////////////////////////////
void drawMainMenu() {
  GLCD.LoadBitmap(MENU); 
  GLCD.DrawRoundRect(4, 5, 59, 52, 4, BLACK);
  menu = MENU_MAIN;
  subMenu = SUB_MENU_MAIN_NEW_SHOT;
}

void loopMainMenu() {
  if(!btn_enter && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
 
 	//On enter go to the selected menu
  if(btn_enter) {
    if(subMenu == SUB_MENU_MAIN_NEW_SHOT)
      drawNewShotFocalLength();
    else
      drawConfigMenu();
  }
  //Change selected menu option and draw a box around the selected option
  else if(btn_left || btn_right) {
    if(subMenu == SUB_MENU_MAIN_NEW_SHOT) {
      subMenu = SUB_MENU_MAIN_CONFIG;
      GLCD.DrawRoundRect(4, 5, 59, 52, 4, WHITE);
      GLCD.DrawRoundRect(64, 5, 59, 52, 4, BLACK);
    }
    else {
      subMenu = SUB_MENU_MAIN_NEW_SHOT;
      GLCD.DrawRoundRect(4, 5, 59, 52, 4, BLACK);
      GLCD.DrawRoundRect(64, 5, 59, 52, 4, WHITE);      
    }
  }
}


///////////////////////////////////////////////////////
//                  CONFIG MENU - PAGE 1             //
///////////////////////////////////////////////////////
void drawConfigMenu() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Config");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  GLCD.SelectFont(Small);
  GLCD.GotoXY(110, 4);
  GLCD.Puts("1/2");

  //Draw the menu options
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(10, 18);
  GLCD.Puts("Order");

  GLCD.GotoXY(10, 28);
  GLCD.Puts("Overlap");

  GLCD.GotoXY(10, 38);
  GLCD.Puts("Stabilization  Delay");

  GLCD.GotoXY(10, 48);
  GLCD.Puts("Bracketing");

  GLCD.GotoXY(1, 18);
  GLCD.Puts(">");

  menu = MENU_CONFIG;
  subMenu = SUB_MENU_CONFIG_ORDER;
  
}

void loopConfigMenu() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
 
 	//On enter go to the selected menu
  if(btn_enter) {
    if(subMenu == SUB_MENU_CONFIG_ORDER)
      drawConfigOrder();
    if(subMenu == SUB_MENU_CONFIG_OVERLAP)
      drawConfigOverlap();
    if(subMenu == SUB_MENU_CONFIG_STAB_DELAY)
      drawConfigStabDelay();
    if(subMenu == SUB_MENU_CONFIG_BRACKETING)
      drawConfigBracketing();
  }
  else if(btn_back) {
    drawMainMenu();
  }
  //Change selected menu option and draw the select mark in front of it
  else if(btn_up || btn_down) {
    if(btn_down) {
      if(subMenu == SUB_MENU_CONFIG_BRACKETING)
        drawConfigPage2Menu();
      else
        subMenu++;
    }
    else if(btn_up) {
      if(subMenu == SUB_MENU_CONFIG_ORDER) {
        subMenu = SUB_MENU_CONFIG_SHUT_DELAY;
        drawConfigPage2Menu();
      }
      else
        subMenu--;
    }
    if(subMenu >= SUB_MENU_CONFIG_ORDER && 
       subMenu <= SUB_MENU_CONFIG_BRACKETING) {
      GLCD.FillRect(0, 18, 9, 46, WHITE);
      GLCD.GotoXY(1, 8 + (subMenu * 10));
      GLCD.Puts(">");
    }
  }
}


///////////////////////////////////////////////////////
//                  CONFIG MENU - PAGE 2             //
///////////////////////////////////////////////////////
void drawConfigPage2Menu() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Config");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  GLCD.SelectFont(Small);
  GLCD.GotoXY(110, 4);
  GLCD.Puts("2/2");

  //Draw the menu options
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(10, 18);
  GLCD.Puts("Shutter  Delay");

  GLCD.GotoXY(1, 18);
  GLCD.Puts(">");

  menu = MENU_CONFIG_PAGE_2;
  subMenu = SUB_MENU_CONFIG_SHUT_DELAY;
  
}

void loopConfigPage2Menu() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
 
 	//On enter go to the selected menu
  if(btn_enter) {
    if(subMenu == SUB_MENU_CONFIG_SHUT_DELAY)
      drawConfigShutterDelay();
  }
  else if(btn_back) {
    drawMainMenu();
  }
  //Change selected menu option and draw the select mark in front of it
  else if(btn_up || btn_down) {
    if(btn_down) {
      if(subMenu == SUB_MENU_CONFIG_SHUT_DELAY) {
        subMenu = SUB_MENU_CONFIG_ORDER;
        drawConfigMenu();
      }
      else
        subMenu++;
    }
    else if(btn_up) {
      if(subMenu == SUB_MENU_CONFIG_ORDER)
         drawConfigMenu();
      else
        subMenu--;
    }
    if(subMenu >= SUB_MENU_CONFIG_SHUT_DELAY && 
       subMenu <= SUB_MENU_CONFIG_SHUT_DELAY) {
      GLCD.FillRect(0, 18, 9, 46, WHITE);
      GLCD.GotoXY(1, 8 + ((subMenu - 4) * 10));
      GLCD.Puts(">");
    }
  }
}


///////////////////////////////////////////////////////
//              CONFIG MENU - ORDER                  //
///////////////////////////////////////////////////////
void drawConfigOrder() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Order");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  menu = MENU_CONFIG_ORDER;
  oldConfigValue = order;

  GLCD.SelectFont(Normal);
  GLCD.GotoXY(10, 18);
  GLCD.Puts("Rows");

  GLCD.GotoXY(10, 28);
  GLCD.Puts("Columns");

  GLCD.GotoXY(1, 8 + (order * 10));
  GLCD.Puts(">");

}

void loopConfigOrder() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  
  // Save the config and go back
  if(btn_enter) {
    if(order != oldConfigValue) {
      EEPROM_writeAnything(EEPROM_ORDER_ADDR, order);
    }
    
    drawConfigMenu();
  }
  // Change back to the old value and go back
  else if(btn_back) {
    order = oldConfigValue;
    drawConfigMenu();
  }
  else if(btn_up || btn_down) {
    if(order == ORDER_ROWS)
      order = ORDER_COLUMNS;
    else
      order = ORDER_ROWS;
    GLCD.FillRect(0, 18, 9, 46, WHITE);
    GLCD.GotoXY(1, 8 + (order * 10));
    GLCD.Puts(">");
  }
  
  
}

///////////////////////////////////////////////////////
//              CONFIG MENU - OVERLAP                //
///////////////////////////////////////////////////////
void drawConfigOverlap() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Overlap");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  menu = MENU_CONFIG_OVERLAP;
  oldConfigValue = overlap;

	// Draw a % progress bar
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(14, 30);
  GLCD.PrintNumber(overlap);
  GLCD.Puts("  %");
  GLCD.DrawRect(14, 40, 100, 10, BLACK);
  GLCD.FillRect(14, 40, overlap, 10, BLACK);
  
}

void loopConfigOverlap() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;

  if(btn_enter || btn_back)
    menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  else
    menu_debounce_timer = millis() + (BUTTON_DEBOUNCE_TIME / 10);
  
  // Save the config and go back
  if(btn_enter) {
    if(overlap != oldConfigValue) {
      EEPROM_writeAnything(EEPROM_OVERLAP_ADDR, overlap);
    }
    drawConfigMenu();
  }
  // Change back to the old value and go back
  else if(btn_back) {
    overlap = oldConfigValue;
    drawConfigMenu();
  }
  else if(btn_up || btn_left || btn_down || btn_right) {
    if((btn_down || btn_left) && overlap > 5) {
      overlap--;
      GLCD.DrawVertLine(14 + overlap + 1, 41, 8, WHITE);
    }
    else if((btn_up || btn_right) && overlap < 95) {
      overlap++;
      GLCD.DrawVertLine(14 + overlap, 41, 8, BLACK);
    }
    
    // Update the % progress bar
    GLCD.FillRect(14, 30, 30, 9, WHITE);
    GLCD.SelectFont(Normal);
    GLCD.GotoXY(14, 30);
    GLCD.PrintNumber(overlap);
    GLCD.Puts("  %");
  }
  
  
}

///////////////////////////////////////////////////////
//              CONFIG MENU - STAB DELAY             //
///////////////////////////////////////////////////////
void drawConfigStabDelay() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Stabilization  Delay");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  menu = MENU_CONFIG_STAB_DELAY;
  oldConfigValue = stab_delay;
  
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(14, 30);
  GLCD.PrintNumber(stab_delay);
  GLCD.Puts("  sec");

}

void loopConfigStabDelay() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  // Save the config and go back
  if(btn_enter) {
    if(stab_delay != oldConfigValue) {
      EEPROM_writeAnything(EEPROM_STAB_DELAY_ADDR, stab_delay);
    }
    
    drawConfigMenu();
  }
  // Change back to the old value and go back
  else if(btn_back) {
    stab_delay = oldConfigValue;
    drawConfigMenu();
  }
  else if(btn_up || btn_left || btn_down || btn_right) {
    if((btn_down || btn_left) && stab_delay > 0) {
      stab_delay--;
    }
    else if((btn_up || btn_right) && stab_delay < 999) {
      stab_delay++;
    }
    
    GLCD.FillRect(14, 30, 30, 9, WHITE);
    GLCD.SelectFont(Normal);
    GLCD.GotoXY(14, 30);
    GLCD.PrintNumber(stab_delay);
    GLCD.Puts("  sec");
  }  
  
  
}

///////////////////////////////////////////////////////
//              CONFIG MENU - BRACKETING             //
///////////////////////////////////////////////////////
void drawConfigBracketing() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Bracketing");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  menu = MENU_CONFIG_BRACKETING;
  oldConfigValue = bracketing;

  GLCD.SelectFont(Normal);
  GLCD.GotoXY(14, 30);
  GLCD.PrintNumber(bracketing);
  if(bracketing == 1)
    GLCD.Puts("  image");  
  else
    GLCD.Puts("  images");

}

void loopConfigBracketing() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  // Save the config and go back
  if(btn_enter) {
    if(bracketing != oldConfigValue) {
      EEPROM_writeAnything(EEPROM_BRACKETING_ADDR, bracketing);
    }
    drawConfigMenu();
  }
  // Change back to the old value and go back
  else if(btn_back) {
    bracketing = oldConfigValue;
    drawConfigMenu();
  }
  else if(btn_up || btn_left || btn_down || btn_right) {
    if((btn_down || btn_left) && bracketing > 1) {
      bracketing--;
    }
    else if((btn_up || btn_right) && bracketing < 3) {
      bracketing++;
    }
    
    GLCD.FillRect(14, 30, 50, 9, WHITE);
    GLCD.SelectFont(Normal);
    GLCD.GotoXY(14, 30);
    GLCD.PrintNumber(bracketing);
    if(bracketing == 1)
      GLCD.Puts("  image");  
    else
      GLCD.Puts("  images");
  }
  
}


///////////////////////////////////////////////////////
//            CONFIG MENU - SHUTTER DELAY            //
///////////////////////////////////////////////////////
void drawConfigShutterDelay() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Shutter  Delay");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  menu = MENU_CONFIG_SHUTTER_DELAY;
  oldConfigValue = shutter_delay;
  
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(14, 30);
  GLCD.PrintNumber(shutter_delay * 100);
  GLCD.Puts("  milli sec");

}

void loopConfigShutterDelay() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  // Save the config and go back
  if(btn_enter) {
    if(shutter_delay != oldConfigValue) {
      EEPROM_writeAnything(EEPROM_SHUTTER_DELAY_ADDR, shutter_delay);
    }
    
    drawConfigMenu();
  }
  // Change back to the old value and go back
  else if(btn_back) {
    shutter_delay = oldConfigValue;
    drawConfigPage2Menu();
  }
  else if(btn_up || btn_left || btn_down || btn_right) {
    if((btn_down || btn_left) && shutter_delay > 0) {
      shutter_delay--;
    }
    else if((btn_up || btn_right) && shutter_delay < 999) {
      shutter_delay++;
    }
    
    GLCD.FillRect(14, 30, 70, 9, WHITE);
    GLCD.SelectFont(Normal);
    GLCD.GotoXY(14, 30);
    GLCD.PrintNumber(shutter_delay * 100);
    GLCD.Puts("  milli  sec");
  }  
  
  
}


///////////////////////////////////////////////////////
//           NEW SHOT MENU - FOCAL LENGTH            //
///////////////////////////////////////////////////////
void drawNewShotFocalLength() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Focal  Length");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  GLCD.SelectFont(Small);
  GLCD.GotoXY(110, 4);
  GLCD.Puts("1/4");
  menu = MENU_NEW_SHOT_FOCAL_LENGTH;

//Screen looks like this:
// > Lens:         300 mm 
//   In 35mm Film: 450 mm
//   Width Angle:  8.3 *
//   Height Angle: 6.2 *

  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.Puts(">");
  GLCD.GotoXY(10, 18);
  GLCD.Puts("Lens:");
  GLCD.GotoXY(10, 28);
  GLCD.Puts("In 35mm Film:");
  GLCD.GotoXY(10, 38);
  GLCD.Puts("Width Angle:");
  GLCD.GotoXY(10, 48);
  GLCD.Puts("Height Angle:");
  drawFocalLengthValues();
  
  oldConfigValue = focal_length;
  
  //Init the Merlin mount and reset start/end position.
  merlin.init();
  start_pos_x = merlin.readAxisPosition(AXIS_YAW);
  start_pos_y = merlin.readAxisPosition(AXIS_PITCH);
  end_pos_x = start_pos_x;
  end_pos_y = start_pos_y;
  
}

void loopNewShotFocalLength() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;

  if(btn_enter || btn_back)
    menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  else
    menu_debounce_timer = millis() + (BUTTON_DEBOUNCE_TIME / 4);

  if(btn_enter) {
    if(focal_length != oldConfigValue) {
      EEPROM_writeAnything(EEPROM_FOCAL_LENGTH_ADDR, focal_length);
    }
    
    focal_length_angle_widht = focalLengthToAngle(focal_length, SENSOR_WIDTH);
    focal_length_angle_height = focalLengthToAngle(focal_length, SENSOR_HEIGHT);
    drawNewShotStartPos();
  }
  else if(btn_back) {
    drawMainMenu();
  }
  else if(btn_up || btn_left || btn_down || btn_right) {
    if((btn_down || btn_left) && focal_length > 1) {
      focal_length--;
    }
    else if((btn_up || btn_right) && focal_length < 9999) {
      focal_length++;
    }
    
    drawFocalLengthValues();
  }
  
}

// Update the focal length screen
void drawFocalLengthValues() {
  GLCD.FillRect(75, 18, 50, 42, WHITE);
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(75, 18);
  GLCD.PrintNumber(focal_length);
  GLCD.Puts("  mm");
    
  GLCD.GotoXY(75, 28);
  GLCD.PrintNumber(focal_length * 1.5);
  GLCD.Puts("  mm");
    
  double angle = focalLengthToAngle(focal_length, SENSOR_WIDTH);
  GLCD.GotoXY(75, 38);
  GLCD.PrintDouble(angle, 100);
  GLCD.Puts("  ^");

  angle = focalLengthToAngle(focal_length, SENSOR_HEIGHT);
  GLCD.GotoXY(75, 48);
  GLCD.PrintDouble(angle, 100);
  GLCD.Puts("  ^");
}

///////////////////////////////////////////////////////
//           NEW SHOT MENU - START POSITION          //
///////////////////////////////////////////////////////
void drawNewShotStartPos() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Start  Position");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  GLCD.SelectFont(Small);
  GLCD.GotoXY(110, 4);
  GLCD.Puts("2/4");
  
  menu = MENU_NEW_SHOT_START_POS;
  
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.Puts("Width:");
  GLCD.GotoXY(1, 28);
  GLCD.Puts("Heigth:");
  GLCD.GotoXY(1, 38);
  GLCD.Puts("Total:");
  drawNumberOfImagesValues();
}

void loopNewShotStartPos() {
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  if(btn_enter) {
    // Move Merlin mount the end position 
    GLCD.FillRect(30, 25, 50, 15, WHITE);
    GLCD.DrawRect(30, 25, 50, 15, BLACK);
    GLCD.SelectFont(Normal);
    GLCD.GotoXY(34, 28);
    GLCD.Puts("Moving...");
    merlin.driveToPositionBothAxis(end_pos_x, end_pos_y, SYNC_SYNCHRONY);
    
    drawNewShotEndPos();
  }
  else if(btn_back) {
    drawNewShotFocalLength();
  }
  
  int old_status_pitch = status_pitch;
  int old_status_yaw = status_yaw;
  
  // Start to move the Merlin mount
  if(btn_up) {
    if(status_pitch != DIR_UP) {
      merlin.startMoving(AXIS_PITCH, DIR_UP);
      status_pitch = DIR_UP;
    }
  }
  else if(btn_down) {
    if(status_pitch != DIR_DOWN) {
      merlin.startMoving(AXIS_PITCH, DIR_DOWN);
      status_pitch = DIR_DOWN;
    }
  }
  // Stop if the button is released
  else if(status_pitch != DIR_STOPPED) {
    merlin.stopMoving(AXIS_PITCH);
    status_pitch = DIR_STOPPED;
  }
  
	// Start to move the Merlin mount
  if(btn_left) {
    if(status_yaw != DIR_LEFT) {
      merlin.startMoving(AXIS_YAW, DIR_LEFT);
      status_yaw = DIR_LEFT;
    }
  }
  else if(btn_right) {
    if(status_yaw != DIR_RIGHT) {
      merlin.startMoving(AXIS_YAW, DIR_RIGHT);
      status_yaw = DIR_RIGHT;
    }
  }
  // Stop if the button is released
  else if(status_yaw != DIR_STOPPED) {
    merlin.stopMoving(AXIS_YAW);
    status_yaw = DIR_STOPPED;
  }
  
  if(old_status_pitch == status_pitch && status_pitch == DIR_STOPPED && 
      old_status_yaw == status_yaw && status_yaw == DIR_STOPPED)
    return;
  if(millis() < menu_update_timer)
    return;
  menu_update_timer = millis() + 600;
  
  start_pos_x = merlin.readAxisPosition(AXIS_YAW);
  start_pos_y = merlin.readAxisPosition(AXIS_PITCH);
  drawNumberOfImagesValues();
}


void drawNumberOfImagesValues() {
//The screen looks like this:
//  Width:   10*    100 Images
//  Heigth:  10*    20 Images
//  Total:          200 Images
  GLCD.FillRect(40, 18, 88, 42, WHITE);
  GLCD.SelectFont(Normal);

  double angleWidth = 0.0;
  double angleHeight = 0.0;
  
  // Calculate the field of view
  if(start_pos_x > end_pos_x)
    angleWidth = abs(merlin.toAngle(start_pos_x, end_pos_x));
  else 
    angleWidth = abs(merlin.toAngle(end_pos_x, start_pos_x));
  if(start_pos_y > end_pos_y)
    angleHeight = abs(merlin.toAngle(start_pos_y, end_pos_y));
  else 
    angleHeight = abs(merlin.toAngle(end_pos_y, start_pos_y));

  
  GLCD.GotoXY(40, 18);
  GLCD.PrintDouble(angleWidth, 100);
  GLCD.Puts("  ^");
    
  GLCD.GotoXY(40, 28);
  GLCD.PrintDouble(angleHeight, 100);
  GLCD.Puts("  ^");
  
  // Calculate the number of photos needed including overlap
  images_horizontal = lround(ceil(angleWidth / focal_length_angle_widht * (((double)overlap / 100.0) + 1)));
  if(images_horizontal == 0)
    images_horizontal = 1;
  GLCD.GotoXY(80, 18);
  GLCD.PrintNumber(images_horizontal);
  GLCD.Puts("  images");

  images_vertical = lround(ceil(angleHeight / focal_length_angle_height * (((double)overlap / 100.0) + 1)));
  if(images_vertical == 0)
    images_vertical = 1;
  GLCD.GotoXY(80, 28);
  GLCD.PrintNumber(images_vertical);
  GLCD.Puts("  images");

  images_total = images_horizontal * images_vertical;
  GLCD.GotoXY(80, 38);
  GLCD.PrintNumber(images_total);
  GLCD.Puts("  images");
}


///////////////////////////////////////////////////////
//           NEW SHOT MENU - END POSITION            //
///////////////////////////////////////////////////////
void drawNewShotEndPos() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("End  Position");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  GLCD.SelectFont(Small);
  GLCD.GotoXY(110, 4);
  GLCD.Puts("3/4");
  
  menu = MENU_NEW_SHOT_END_POS;
  
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.Puts("Width:");
  GLCD.GotoXY(1, 28);
  GLCD.Puts("Heigth:");
  GLCD.GotoXY(1, 38);
  GLCD.Puts("Total:");
  drawNumberOfImagesValues();
}

void loopNewShotEndPos() {
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  if(btn_enter) {
    drawNewShotReady();
  }
  else if(btn_back) {
    // Move Merlin mount the start position 
    GLCD.FillRect(30, 25, 50, 15, WHITE);
    GLCD.DrawRect(30, 25, 50, 15, BLACK);
    GLCD.SelectFont(Normal);
    GLCD.GotoXY(34, 28);
    GLCD.Puts("Moving...");
    merlin.driveToPositionBothAxis(start_pos_x, start_pos_y, SYNC_SYNCHRONY);

    drawNewShotStartPos();
  }
  
  int old_status_pitch = status_pitch;
  int old_status_yaw = status_yaw;
  
  // Start to move the Merlin mount
  if(btn_up) {
    if(status_pitch != DIR_UP) {
      merlin.startMoving(AXIS_PITCH, DIR_UP);
      status_pitch = DIR_UP;
    }
  }
  else if(btn_down) {
    if(status_pitch != DIR_DOWN) {
      merlin.startMoving(AXIS_PITCH, DIR_DOWN);
      status_pitch = DIR_DOWN;
    }
  }
  // Stop if the button is released
  else if(status_pitch != DIR_STOPPED) {
    merlin.stopMoving(AXIS_PITCH);
    status_pitch = DIR_STOPPED;
  }
  
	// Start to move the Merlin mount
  if(btn_left) {
    if(status_yaw != DIR_LEFT) {
      merlin.startMoving(AXIS_YAW, DIR_LEFT);
      status_yaw = DIR_LEFT;
    }
  }
  else if(btn_right) {
    if(status_yaw != DIR_RIGHT) {
      merlin.startMoving(AXIS_YAW, DIR_RIGHT);
      status_yaw = DIR_RIGHT;
    }
  }
  // Stop if the button is released
  else if(status_yaw != DIR_STOPPED) {
    merlin.stopMoving(AXIS_YAW);
    status_yaw = DIR_STOPPED;
  }
  
  if(old_status_pitch == status_pitch && status_pitch == DIR_STOPPED && 
      old_status_yaw == status_yaw && status_yaw == DIR_STOPPED)
    return;
  if(millis() < menu_update_timer)
    return;
  menu_update_timer = millis() + 600;
  
  end_pos_x = merlin.readAxisPosition(AXIS_YAW);
  end_pos_y = merlin.readAxisPosition(AXIS_PITCH);
  drawNumberOfImagesValues();
}


///////////////////////////////////////////////////////
//               NEW SHOT MENU - READY               //
///////////////////////////////////////////////////////
void drawNewShotReady() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Ready  to  Start?");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  GLCD.SelectFont(Small);
  GLCD.GotoXY(110, 4);
  GLCD.Puts("4/4");
  
  menu = MENU_NEW_SHOT_READY;

//The screen looks like this:
//  100 x 10 x 2 =   400 Images
//  Order:           Columns
//  Focal Length:    300 mm
//  Overlap:         25 %

  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.PrintNumber(images_horizontal);
  GLCD.Puts("  x  ");
  GLCD.PrintNumber(images_vertical);
  GLCD.Puts("  x  ");
  GLCD.PrintNumber(bracketing);
  GLCD.Puts("  =  ");
  GLCD.GotoXY(75, 18);
  GLCD.PrintNumber(images_total);
 
  
  GLCD.GotoXY(1, 28);
  GLCD.Puts("Order:");
  GLCD.GotoXY(75, 28);
  if(order == ORDER_ROWS)
    GLCD.Puts("Rows");
  else
    GLCD.Puts("Columns");
 
  GLCD.GotoXY(1, 38);
  GLCD.Puts("Focal Length:");
  GLCD.GotoXY(75, 38);
  GLCD.PrintNumber(focal_length);
  GLCD.Puts("  mm");
  
  GLCD.GotoXY(1, 48);
  GLCD.Puts("Overlap:");
  GLCD.GotoXY(75, 48);
  GLCD.PrintNumber(overlap);
  GLCD.Puts("  %");

}

void loopNewShotReady() {
  if(!btn_enter && !btn_back)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  if(btn_enter) {
  	//Starting shooting from start position
    drawProgress(true);
  }
  else if(btn_back) {
    drawNewShotEndPos();
  }
}






///////////////////////////////////////////////////////
//                       PROGRESS                    //
///////////////////////////////////////////////////////
void drawProgress(boolean restart) {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Shooting...");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  
  menu = MENU_PROGRESS;
  GLCD.SelectFont(Normal);
  
  if(restart) {
    GLCD.GotoXY(14, 45);
    GLCD.Puts("Moving  to  Start  Position...");
    
    //Start position must always be the min value so the position always increase to the next shot
    long min_pos = min(start_pos_x, end_pos_x);
    long max_pos = max(start_pos_x, end_pos_x);
    start_pos_x = min_pos;
    end_pos_x = max_pos;
    min_pos = min(start_pos_y, end_pos_y);
    max_pos = max(start_pos_y, end_pos_y);
    start_pos_y = min_pos;
    end_pos_y = max_pos;

    
    current_image = 1;
    current_horizontal = 1;
    current_vertical = 1;
    shooting_start_time = millis();
    progress_state = PROGRESS_MOVING;
    merlin.driveToPositionBothAxis(start_pos_x, start_pos_y, SYNC_ASYNCHRONY);
  }

  
  GLCD.GotoXY(14, 18);
  GLCD.Puts("Image  ");
  GLCD.PrintNumber(current_image);
  GLCD.Puts("  of  ");
  GLCD.PrintNumber(images_total);
  GLCD.DrawRect(14, 30, 100, 10, BLACK);
  GLCD.FillRect(14, 30, lround(ceil((((double)current_image)/ (double)images_total) * 100.0)), 10, BLACK);

  
}

void loopProgress() {
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  if(btn_enter) {
    drawPause();
    return;
  }
  else if(btn_back) {
    drawCancel();
    return;
  }
  
  if(millis() > progress_timer) {
    progress_timer = millis() + 100;
    
    //Should it go to the next progress state?
    if(progress_state == PROGRESS_MOVING) {
      if(merlin.readAxisStatus(AXIS_YAW) == STATUS_STOPPED && merlin.readAxisStatus(AXIS_PITCH) == STATUS_STOPPED) {
         //Start the stabilisation delay
         progress_state = PROGRESS_WAIT;
         wait_timer = millis() + (stab_delay * 1000);
         GLCD.SelectFont(Normal);
         GLCD.FillRect(1, 45, 128, 10, WHITE);
         GLCD.GotoXY(14, 45);
         GLCD.Puts("Stabilization  Delay...");
       }
     }
     else if(progress_state == PROGRESS_WAIT) {
       if(millis() > wait_timer) {
         //Start to shot
         progress_state = PROGRESS_SHOOTING;
         GLCD.SelectFont(Normal);
         GLCD.FillRect(1, 45, 128, 10, WHITE);
         GLCD.GotoXY(14, 45);
         GLCD.Puts("Shooting...");
         for(int i = 0; i < bracketing; i++) {
           merlin.takePicture(150);
           delay(100);
         }
         // Was it the last picture?
         if(current_image == images_total) {
           drawFinished();
           return;
         }
       }
     }
     else if(progress_state == PROGRESS_SHOOTING) { 
       //Start the shutter delay
         progress_state = PROGRESS_WAIT_SHUT;
         wait_timer = millis() + (shutter_delay * 100);
         GLCD.SelectFont(Normal);
         GLCD.FillRect(1, 45, 128, 10, WHITE);
         GLCD.GotoXY(14, 45);
         GLCD.Puts("Shutter  Delay...");
     }
     else if(progress_state == PROGRESS_WAIT_SHUT) { 
       if(millis() > wait_timer) {
         //Move to the next position
         progress_state = PROGRESS_MOVING;
         GLCD.SelectFont(Normal);
         GLCD.FillRect(1, 45, 128, 10, WHITE);
         GLCD.GotoXY(14, 45);
         GLCD.Puts("Moving...");
         current_image++;
         if(order == ORDER_ROWS) {
           if(current_vertical % 2 == 1) {  // Move odd rows left to right
             current_horizontal++;
             if(current_horizontal > images_horizontal) {
               current_vertical++;
               current_horizontal = images_horizontal;
             }
           }
           else {  // Move even rows right to left
             current_horizontal--;
             if(current_horizontal < 1) {
               current_vertical++;
               current_horizontal = 1;
             }
           }
         }
         else {
           if(current_horizontal % 2 == 1) {  // Move odd cols left to right
             current_vertical++;
             if(current_vertical > images_vertical) {
               current_horizontal++;
               current_vertical = images_vertical;
             }
           }
           else {  // Move even cols right to left
             current_vertical--;
             if(current_vertical < 1) {
               current_horizontal++;
               current_vertical = 1;
             }
           }
         }
       
         //Update the progress
         GLCD.FillRect(1, 18, 128, 10, WHITE);
         GLCD.GotoXY(14, 18);
         GLCD.Puts("Image  ");
         GLCD.PrintNumber(current_image);
         GLCD.Puts("  of  ");
         GLCD.PrintNumber(images_total);
    
         GLCD.FillRect(14, 30, lround(ceil((((double)current_image) / (double)images_total) * 100.0)), 10, BLACK);
       
         long pos_x = 0;
         long pos_y = 0;
         if(current_horizontal == 1)
           pos_x = start_pos_x;
         else
           pos_x = start_pos_x + lround((((double)current_horizontal - 1.0) * (double)merlin.fromAngle(focal_length_angle_widht)) / (((double)overlap / 100.0) + 1.0));
       
         if(current_vertical == 1)
           pos_y = start_pos_y;
         else
           pos_y = start_pos_y + lround((((double)current_vertical - 1.0) * (double)merlin.fromAngle(focal_length_angle_height)) / (((double)overlap / 100.0) + 1.0));
         merlin.driveToPositionBothAxis(pos_x, pos_y, SYNC_ASYNCHRONY);   
      }
    }
  } 
}


///////////////////////////////////////////////////////
//                       FINISHED                    //
///////////////////////////////////////////////////////
void drawFinished() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Finished");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  
  menu = MENU_FINISHED;

//The sreen looks liek this:
//  100 x 10 x 2 = 400 Images Shot
//  in 100 minutes and 2 seconds.
                    
  int minutes = ((millis() - shooting_start_time) / 1000) / 60;
  int seconds = ((millis() - shooting_start_time) / 1000) % 60;

  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.PrintNumber(images_horizontal);
  GLCD.Puts("  x  ");
  GLCD.PrintNumber(images_vertical);
  GLCD.Puts("  x  ");
  GLCD.PrintNumber(bracketing);
  GLCD.Puts("  =  ");
  GLCD.PrintNumber(images_total);
  GLCD.Puts("  Images  shot");
  GLCD.GotoXY(1, 28);
  GLCD.Puts("in  ");
  if(minutes > 0) {
    GLCD.PrintNumber(minutes);
    GLCD.Puts("  min  and  ");
  }
  GLCD.PrintNumber(seconds);
  GLCD.Puts("  sec.");
  
  playSuccessMelody();
}

void loopFinished() {
  if(!btn_enter && !btn_back)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  if(btn_enter || btn_back) {
    drawMainMenu();
  } 
}


///////////////////////////////////////////////////////
//                        CANCEL                     //
///////////////////////////////////////////////////////
void drawCancel() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Cancel?");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  
  menu = MENU_CANCEL;
  
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.Puts("Are  you  sure  you  want");
  GLCD.GotoXY(1, 28);
  GLCD.Puts("to  cancel?");
}

void loopCancel() {
  if(!btn_enter && !btn_back)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;

  if(btn_enter) {
    drawMainMenu();
  }
  else if(btn_back) {
  	//go back to progress but don't start over
    drawProgress(false);
  }
}


///////////////////////////////////////////////////////
//                         PAUSE                     //
///////////////////////////////////////////////////////
void drawPause() {
  GLCD.ClearScreen();
  GLCD.SelectFont(Header);
  GLCD.GotoXY(2, 1);
  GLCD.Puts("Pause");
  GLCD.DrawHoriLine(0, 14, 128, BLACK);
  
  menu = MENU_PAUSE;
  
  pause_image = current_image;
  
  GLCD.SelectFont(Normal);
  GLCD.GotoXY(1, 18);
  GLCD.Puts("Current  Image  is  ");
  GLCD.PrintNumber(current_image);
  GLCD.Puts("  of  ");
  GLCD.PrintNumber(images_total);
  GLCD.GotoXY(1, 28);
  GLCD.Puts("Go  Directly  to:");
  GLCD.GotoXY(70, 28);
  GLCD.PrintNumber(pause_image);
}

void loopPause() {
  if(!btn_enter && !btn_back && !btn_up && !btn_down && !btn_left && !btn_right)
    return;
  if(millis() < menu_debounce_timer)
    return;
  menu_debounce_timer = millis() + BUTTON_DEBOUNCE_TIME;
  //On the pause screen can you go back or forward
  if(btn_enter) {
    current_image = pause_image - 1;
    if(order == ORDER_ROWS) {
      current_vertical = current_image / images_horizontal;
      current_horizontal = current_image % images_horizontal;
    }
    else {
      current_vertical = current_image % images_vertical;
      current_horizontal = current_image / images_vertical;
      
    }
    progress_state = PROGRESS_SHOOTING;
    drawProgress(false);
  }
  else if(btn_back) {
    drawProgress(false);
  }
  else if(btn_up || btn_left || btn_right || btn_down) {
    if(btn_up || btn_right && pause_image <= images_total)
      pause_image++;
    else if(pause_image > 0)
      pause_image--;
    
    GLCD.FillRect(70, 28, 58, 10, WHITE);
    GLCD.GotoXY(70, 28);
    GLCD.PrintNumber(pause_image);
  }
  
}



///////////////////////////////////////////////////////
///////////////////////////////////////////////////////
//                 GENERAL FUNCTIONS                 //
///////////////////////////////////////////////////////
///////////////////////////////////////////////////////
void updateButtonsState() {
  //Shift in and update all buttons
  digitalWrite(BUTTONS_CLOCK_PIN, HIGH);

  //Pulse the latch pin, set it to 1 to collect parallel data
  digitalWrite(BUTTONS_LATCH_PIN, LOW);
  //set it to 1 to collect parallel data, wait
  delayMicroseconds(20);
  //set it to 0 to transmit data serially  
  digitalWrite(BUTTONS_LATCH_PIN, HIGH);

  byte switchVar = shiftIn(BUTTONS_DATA_PIN, BUTTONS_CLOCK_PIN);
  if(switchVar & 1) btn_back = true; else btn_back = false;
  if(switchVar & 2) btn_enter = true; else btn_enter = false;
  if(switchVar & 4) btn_down = true; else btn_down = false;
  if(switchVar & 8) btn_up = true; else btn_up = false;
  if(switchVar & 16) btn_left = true; else btn_left = false;
  if(switchVar & 32) btn_right = true; else btn_right = false;
    

  digitalWrite(BUTTONS_LATCH_PIN, LOW);
}

////// ----------------------------------------shiftIn function
///// just needs the location of the data pin and the clock pin
///// it returns a byte with each bit in the byte corresponding
///// to a pin on the shift register. leftBit 7 = Pin 7 / Bit 0= Pin 0
byte shiftIn(int myDataPin, int myClockPin) { 
  int i;
  int temp = 0;
  byte myDataIn = 0;

//we will be holding the clock pin high 8 times (0,..,7) at the
//end of each time through the for loop

//The register transmits the information about the pins from pin 7 to pin 0
//so that is why our function counts down
  for (i=7; i>=0; i--) {
   temp = digitalRead(myDataPin);
    if (temp) {
      //set the bit to 0 no matter what
      myDataIn = myDataIn | (1 << i);
    }

    digitalWrite(myClockPin, LOW);
    delayMicroseconds(1);
    digitalWrite(myClockPin, HIGH);
  }
  return myDataIn;
}


void playTone(int tone, int duration) {
  for (long i = 0; i < duration * 1000L; i += tone * 2) {
    digitalWrite(SPEEAKER_PIN, HIGH);
    delayMicroseconds(tone);
    digitalWrite(SPEEAKER_PIN, LOW);
    delayMicroseconds(tone);
  }
}

void playNote(char note, int duration) {
  char names[] = { 'c', 'd', 'e', 'f', 'g', 'a', 'b', 'C' };
  int tones[] = { 1915, 1700, 1519, 1432, 1275, 1136, 1014, 956 };

  // play the tone corresponding to the note name
  for (int i = 0; i < 8; i++) {
    if (names[i] == note) {
      playTone(tones[i], duration);
    }
  }
}

void playSuccessMelody() {
  playNote('e', 800);
  delay(80);
  playNote('e', 800);
  delay(80);
  playNote('e', 800);
  
}


// Calculate the angle from focal length
double focalLengthToAngle(int fl, double dimension) {
  return atan(dimension / (2 * (double)fl)) * 360 / M_PI;  
}


